module DrugParams

using Distributions,DistributionsExtended

export Results,Params,Assets,Bonds,Simu,Ce,StateDict,ParamDict,StrucExit

export mergeresults,mergeParams


type Results
	cash_begin_per
    cash     
    payoff    
	reserve
	reserve_cash
    weight_tranche
    weight_cash
    cash_for_investment 
	cash_for_buying
    A1_bals           
    A2_bals          
    amort_sched     
    A1_payment     
    A2_payment    
    IC_ratio     
    ROE   
    IRR 
    sale_phases
    withdrawal_phases
    sale_times      
    withdrawal_times
    extra_cash_for_coverage
    cash_from_milestones
    servicing             
    compounds_to_fund    
    compounds_funded     
    compounds_initially_bought
    compounds_current
    params            
    final_sale_values 
    compounds_left   
    A1_defaults     
    A2_defaults    
    svc_defaults  
	guarantee_begin_per 
    guarantee_draw     
    drew_on_guarantee 
end

function mergeresults(r1::Results,r2::Results)
	return Results(
		vcat(r1.cash_begin_per,r2.cash_begin_per),
		vcat(r1.cash,r2.cash),
        vcat(r1.payoff,r2.payoff),
		vcat(r1.reserve,r2.reserve),
		vcat(r1.reserve_cash,r2.reserve_cash),
        vcat(r1.weight_tranche,r2.weight_tranche),
        vcat(r1.weight_cash,r2.weight_cash),
		vcat(r1.cash_for_investment,r2.cash_for_investment),
		vcat(r1.cash_for_buying,r2.cash_for_buying),
		vcat(r1.A1_bals,r2.A1_bals),
		vcat(r1.A2_bals,r2.A2_bals),
		r1.amort_sched,
		vcat(r1.A1_payment,r2.A1_payment),
		vcat(r1.A2_payment,r2.A2_payment),
		r1.IC_ratio,
		vcat(r1.ROE,r2.ROE),
        vcat(r1.IRR,r2.IRR),
		vcat(r1.sale_phases,r2.sale_phases),
		vcat(r1.withdrawal_phases,r2.withdrawal_phases),
		vcat(r1.sale_times,r2.sale_times),
		vcat(r1.withdrawal_times,r2.withdrawal_times),
		vcat(r1.extra_cash_for_coverage,r2.extra_cash_for_coverage),
        vcat(r1.cash_from_milestones,r2.cash_from_milestones),
		vcat(r1.servicing,r2.servicing),
		vcat(r1.compounds_to_fund,r2.compounds_to_fund),
		vcat(r1.compounds_funded,r2.compounds_funded),
		(r1.params.simu.NSIMUS*r1.compounds_initially_bought+r2.params.simu.NSIMUS*r2.compounds_initially_bought)/(r1.params.simu.NSIMUS+r2.params.simu.NSIMUS),
		(r1.params.simu.NSIMUS*r1.compounds_current+r2.params.simu.NSIMUS*r2.compounds_current)/(r1.params.simu.NSIMUS+r2.params.simu.NSIMUS),
		mergeParams(r1.params,r2.params),
		vcat(r1.final_sale_values,r2.final_sale_values),
		vcat(r1.compounds_left,r2.compounds_left),
		vcat(r1.A1_defaults,r2.A1_defaults),
		vcat(r1.A2_defaults,r2.A2_defaults),
		vcat(r1.svc_defaults,r2.svc_defaults),
		vcat(r1.guarantee_begin_per,r2.guarantee_begin_per),
		vcat(r1.guarantee_draw,r2.guarantee_draw),
		vcat(r1.drew_on_guarantee,r2.drew_on_guarantee))
end

type StateDict
	DSC::Number
	PRE::Number
	P1::Number
	P2::Number
	P3::Number
	NDA::Number
	APP::Number
	SLD::Number
end
StateDict() = StateDict(0)
StateDict(x::StateDict) = (x.DSC,x.PRE,x.P1,x.P2,x.P3,x.NDA,x.APP,x.SLD)
StateDict(x) = StateDict(x,x,x,x,x,x,x,x)

type StrucExit
    valueDist::Array{UnivariateDistribution,1}
    midValueDist::Array{UnivariateDistribution,1}
    milestone::Array{UnivariateDistribution,2}
    milestoneMid::Array{UnivariateDistribution,2}
end
StrucExit()=StrucExit([],[],[],[])
StrucExit(n::Integer)=StrucExit(Array(UnivariateDistribution,n),
    Array(UnivariateDistribution,n),
    Array(UnivariateDistribution,n,n),
    Array(UnivariateDistribution,n,n))

type Assets
	rho::Float64
	trans_prob::Array{Float64,2}
	trans_prob_col::StateDict
    upfront::Array{Float64,1}
    milestone::Array{Float64,1}
    valueDist::Array{UnivariateDistribution,1}
    midValueDist::Array{UnivariateDistribution,1}
    costDist::Array{UnivariateDistribution,1}
    probUpfrontExit::Array{Float64,1}
    strucExit::StrucExit
    budget::Array{Float64,1}
	sell_in_phase::Array
    sale_delay::Array{Float64,1}
	sale_time::Array{Float64,1}
	ratio_unchanged::Number
    equity_stake::Number
    timeMode::String
	timeDist::Array{UnivariateDistribution,1}
	steady::Array{Float64,1}
    science_hitfactor::Number
    science_hitrate::Number
end

type Bonds{T}
    capital_structure::Array{Float64,1}
    IC_pers::Int64
    interest_coverage::Array{Float64,1}
    amort_timing::Array{T}
	coupon::Array{Float64,1}
    servicing_rate::FloatingPoint 
    svc_accrual_int_rate::FloatingPoint 
    cash_accrual_int_rate::FloatingPoint 
	nominal::Array{Float64,1}
end
Bonds(x::Bonds)=Bonds(x.capital_structure,x.IC_pers,x.interest_coverage,x.amort_timing,x.coupon,
	x.servicing_rate,x.svc_accrual_int_rate,x.cash_accrual_int_rate,x.nominal)
type Ce
    ce_amt::Number
    ce_repay::Bool   
    ce_premium::Number 
    
end
Ce(x::Ce) = Ce(x.ce_amt,x.ce_repay,x.ce_premium)
type Simu
    NSIMUS::Integer 
    TIMESTEPS::Integer
	STEPS_YR::Integer
    initial_cash::Number
    initial_compounds::Array{Integer,3} 
	reserve_flag::Bool
	tranche_weights::Array{Float64,2}
	tranche_cashweights::Array{Float64,1}
    tranche_capratio::Array{Float64,1}
	fixedcost::Number
    MINPAYOFFTIME::Integer
    performance_fee::Number
    rhos::Array
    successDist::Array{UnivariateDistribution,1}
    vCorrPeriods::Integer
end

Simu(x::Simu)=Simu(x.NSIMUS,x.TIMESTEPS,x.STEPS_YR,x.initial_cash,x.initial_compounds,x.reserve_flag,
    x.tranche_weights,x.tranche_cashweights,x.tranche_capratio,x.fixedcost,x.MINPAYOFFTIME,x.performance_fee,
    x.rhos,x.successDist)


type Params
	simu::Simu
	assets::Array{Assets,1}
	bonds::Bonds
	ce::Ce
end

function mergeParams(params1::Params,params2::Params)
	params = params1
	params.simu.NSIMUS+=params2.simu.NSIMUS
	return params
end

end
