#Input:
#@results is of type struct and is a simulation results summary object
#which is the resultant  output of the simulate_CFs_fn function 
module PlotDiagnostics

using Winston
using UtilityFunctions

export plot_standard_diagnostics_fn

function plot_standard_diagnostics_fn(results,plotfile)
    SMALLFONTSZ = 10
    BIGFONTSZ   = 12
	quants = [0,0.05,0.1,0.25,0.5,0.75]
	colors = ["red", "orange", "magenta", "blue", "green", "black"]
	qcash=mapslices(x->quantile(x,quants),results.cash,1)
	qA1=mapslices(x->quantile(x,quants),results.A1_bals,1)
	qA2=mapslices(x->quantile(x,quants),results.A2_bals,1)
 
	t1 = Table(3,2)
	
	p = FramedPlot()
	setattr(p, "title", "Cash - Quantiles")
    setattr(p, "xlabel","Period")
	setattr(p, "ylabel","Equity (\$ millions)")
	print(size(qcash))
	for q = 1:length(quants)
		add(p, Curve([1:length(qcash[q,:])],qcash[q,:],"color",colors[q]))
	end	
	t1[1,1]=p
	
   	ROE_raw = results.ROE;
	NYEARS  = length(results.cash[1,:])/results.params.simu.STEPS_YR
	NEQUITY = length(results.params.simu.tranche_cashweights)
    ROE_annualized = ((1+ROE_raw).^(1/NYEARS))-1

	p = FramedPlot()

	for i =1:NEQUITY
        h = Histogram(hist(ROE_annualized[:,i],100) ...,"color",colors[i])
        setattr(h,"label","ROE annualized $i")
    	add(p,h)
        h = Histogram(hist(results.IRR[:,i],100) ...,"color",colors[i+NEQUITY])
        setattr(h,"label","IRR annualized $i")
        add(p,h)
    end
    l=Legend(.1,.9,p.content1.components)
    add(p,l)
	setattr(p, "ylabel", "Frequency")
	setattr(p, "title", "ROE Annualized")
	t1[1,2]=p

    p = FramedPlot()
	for q = 1:length(quants)
		add(p, Curve([1:length(qA1[q,:])],qA1[q,:],"color",colors[q]))
	end	
	setattr(p, "xlabel","Period")
	setattr(p, "ylabel","Debt (\$ millions)")
	setattr(p, "title","A1 balance - Quantiles")
	t1[2,1]=p

    p = FramedPlot()
	for q = 1:length(quants)
		add(p, Curve([1:length(qA2[q,:])],qA2[q,:],"color",colors[q]))
	end	
	setattr(p, "xlabel","Period")
	setattr(p, "ylabel","Debt (\$ millions)")
	setattr(p, "title","A2 balance - Quantiles")
	t1[2,2]=p

    
	p = FramedPlot()
    sale_phases = mean(results.sale_phases,1)
    withdrawal_phases = mean(results.withdrawal_phases,1)
	setattr(p, "xlabel","Clinical Phase")
	setattr(p, "ylabel","Num. Compounds")
	setattr(p, "title","Compounds Sold and Withdrawn")
	a=Points([1:length(sale_phases)],vec(sale_phases),color="green",kind="filled circle")
	setattr(a,label="Sold")
	b=Points([1:length(withdrawal_phases)],vec(withdrawal_phases), color="red")
	setattr(b,label="Withdrawn")
	setattr(p.x1, draw_ticks = false, range = [1.5,7.5],ticks=[2,3,4,5,6,7],ticklabels=["PRE","P1","P2","P3","NDA","APP"])
	l=Legend(.75,.9, {a,b})
	add(p,a,b,l)
	t1[3,1]=p
	
	p = FramedPlot()
  	sale_times       = mean(results.sale_times,1)
    withdrawal_times = mean(results.withdrawal_times,1)
	setattr(p, "xlabel","Period")
	setattr(p, "ylabel","Num. Compounds")
	setattr(p, "title","Compounds Sold and Withdrawn")
	
	add(p, Curve([1:length(sale_times)],vec(sale_times), "color", "green"))
	add(p, Curve([1:length(withdrawal_times)],vec(withdrawal_times), "color", "red"))
	t1[3,2]=p	
	
	Winston.display(t1) 
	savefig(t1,plotfile,"width",1200,"height",1000)
  
end


end

#COPYRIGHT 2012
# This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

