saveResults = true

if saveResults
    using HDF5,JLD
end

using DataFrames
using Gadfly

@everywhere using DrugSimulator
using SummarizeResults

#This code has been prepared to support the simulations presented in the paper "Financing drug discovery for orphan diseases"
#by D. Fagnan, A. Gromatzky, R. Stein, JM. Fernandez, A. Lo published in Drug Discovery Today in Nov 2013.

## USE YOUR OWN PARAMETER FILE HERE
require("tests/init_params_fn_DDT.jl")

function main(args,saveResults)
	filename  = convert(ASCIIString,args[1])
	jldfile   = string("results/",filename,".jld")
	qplotfile = string("plots/",filename,".png")
	splotfile = string("plots/summary",filename,".png")

	##Use parameter file for detailed parameter setup.
	params = init_params_fn()

	##Adjust number of simulations.
	params.simu.NSIMUS = 1000

	## Display progress or debugging level.
	progress = true
	db = 0

	##Run in parallel if multiple processors.
	ncpu = nprocs()
	if ncpu > 1
		params.simu.NSIMUS = div(params.simu.NSIMUS,ncpu-1.0)
		println("Running in parallel...$ncpu $(params.simu.NSIMUS)")
		params_array = [deepcopy(params) for i in 1:ncpu-1]
		temp = pmap(x->simulate_CFs_fn(x,progress,db),params_array)
		@time results = reduce(mergeresults,temp[1],temp[2:ncpu-1])
	else
		@time results = simulate_CFs_fn(params,progress,db)
	end

	#Summarize results and plot visuals.
	plotsummary = true
    #try
	   summarize_results_fn(results,plotsummary,splotfile)
    #catch e
        #println("Error during summarize results...$e")
    #end

    #SAVE RESULTS OBJECT (WARNING - LARGE)
	if saveResults
		@save jldfile results
	end

	##Extra pretty plot of cash quantiles using Gadfly.
	p = [0.01, 0.2:0.2:0.8]

	cashquant = DataFrame(Time=Float64[],Quantile=Float64[],Cash=Float64[])
	for i =1:size(results.cash,2)
		tempquant = DataFrame(Time=i/2*ones(length(p)),Quantile=p,Cash=quantile(results.cash[:,i],p)+1)
		append!(cashquant,tempquant)
	end

	p=Gadfly.plot(cashquant,{:x=>"Time",:y=>"Cash",:color=>"Quantile"},Gadfly.Geom.line,Gadfly.Scale.y_log10,Gadfly.Scale.discrete_color)
	Gadfly.draw(Gadfly.PNG(qplotfile,8inch,6inch),p)
	println("Finished.")

end

#FIX RANDOM SEED
srand(1234)
main(ARGS,saveResults)
