push!(LOAD_PATH,"../")

using DrugSimulator
using FactCheck

#NSIMUS=50000
NSIMUS=1000

#=
##Basic test of geometric time distribution.
facts("Drug discovery Today Geometric") do
    reload("init_params_fn_DDT.jl")
    params=init_params_fn()
	params.simu.NSIMUS=NSIMUS
    results=simulate_CFs_fn(params,false,false)
    
	LIQHORIZON    = results.params.simu.TIMESTEPS+int(ceil(maximum(results.params.assets[1].sale_time)))
    NGUARPERS  = size(results.guarantee_draw,2)
    NYEARS     = LIQHORIZON/results.params.simu.STEPS_YR
	
	ROE_raw        = results.ROE
    ROE_annualized = ((1+ROE_raw).^(1/NYEARS))-1
    ROE_mean       = mean(ROE_raw)
    ROE_mean_a     = mean(ROE_annualized)
    
    @fact ROE_mean   => roughly(2.38,0.02)
    @fact ROE_mean_a => roughly(0.134,0.0015)

    sale_phases = mean(results.sale_phases,1)
    @fact sale_phases[5] => roughly(4.7,0.1)
    @fact sale_phases[4] => roughly(2.2,0.1)
    @fact sale_phases[3] => roughly(0.4,0.05)
end

##Basic test using two (identical) asset types
facts("Drug discovery Today Geometric with Multiple Types") do
    params=init_params_fn()
    TIMESTEPS = size(params.simu.initial_compounds,1)
    NPHASES   = size(params.simu.initial_compounds,2)
    params.simu.initial_compounds = zeros(TIMESTEPS,NPHASES,2)
    params.simu.initial_compounds[1,:,1] = [0, 4, 4, 0, 0, 0, 0]
    params.simu.initial_compounds[1,:,2] = [0, 4, 4, 0, 0, 0, 0]
    params.assets=[params.assets[1];params.assets[1]]

	params.simu.NSIMUS=NSIMUS
    results=simulate_CFs_fn(params,false,false)
	LIQHORIZON    = results.params.simu.TIMESTEPS+int(ceil(maximum(results.params.assets[1].sale_time)))
    NGUARPERS  = size(results.guarantee_draw,2)
    NYEARS     = LIQHORIZON/results.params.simu.STEPS_YR
	
	ROE_raw        = results.ROE
    ROE_annualized = ((1+ROE_raw).^(1/NYEARS))-1
    ROE_mean       = mean(ROE_raw)
    ROE_mean_a     = mean(ROE_annualized)
    
    @fact ROE_mean   => roughly(2.38,0.02)
    @fact ROE_mean_a => roughly(0.134,0.0015)

    sale_phases = mean(results.sale_phases,1)
    @fact sale_phases[5] => roughly(4.7,0.1)
    @fact sale_phases[4] => roughly(2.2,0.1)
    @fact sale_phases[3] => roughly(0.4,0.05)
end

##Basic test using Log-Normal clinical trial times
facts("Drug discovery Today Log-Normal") do
    params=init_params_fn()
	params.simu.NSIMUS=NSIMUS
    NPHASES   = size(params.simu.initial_compounds,2)
    tDistType = "LogNormal"

    params.assets[1].timeMode = "Years"
    for i = 2:(NPHASES-1) #SKIP ABSORBING STATES
        ##Convert from Period to Years
        meanDur = (mean(params.assets[1].timeDist[i])+1)/2.0
        params.assets[1].timeDist[i]=quickTimeDist(tDistType,meanDur)
    end
    
    results    = simulate_CFs_fn(params,false,false)
   	LIQHORIZON    = results.params.simu.TIMESTEPS+int(ceil(maximum(results.params.assets[1].sale_time)))
    NGUARPERS  = size(results.guarantee_draw,2)
    NYEARS     = LIQHORIZON/results.params.simu.STEPS_YR
	
	ROE_raw        = results.ROE
    ROE_annualized = ((1+ROE_raw).^(1/NYEARS))-1
    ROE_mean       = mean(ROE_raw)
    ROE_mean_a     = mean(ROE_annualized)
    
    @fact ROE_mean   => roughly(2.28,0.02)
    @fact ROE_mean_a => roughly(0.133,0.0015)

    sale_phases = mean(results.sale_phases,1)
    @fact sale_phases[5] => roughly(4.0,0.1)
    @fact sale_phases[4] => roughly(3.7,0.1)
    @fact sale_phases[3] => roughly(0.2,0.5)
end
=#
#Basic test using original markov-chain formulation for trial times
facts("Nature Biotech Simulation A using Markov Chain") do
    reload("init_params_fn_NBTx_A.jl")
    params=init_params_fn()
    params.simu.NSIMUS=NSIMUS
    results=simulate_CFs_fn(params,false,false)
 	LIQHORIZON    = results.params.simu.TIMESTEPS+int(ceil(maximum(results.params.assets[1].sale_time)))
    NGUARPERS  = size(results.guarantee_draw,2)
    NYEARS     = LIQHORIZON/results.params.simu.STEPS_YR
	
    ROE_raw        = results.ROE
    ROE_annualized = ((1+ROE_raw).^(1/NYEARS))-1
    ROE_mean       = mean(ROE_raw)
    ROE_mean_a     = mean(ROE_annualized)
    
    
    @fact ROE_mean   => roughly(1.45,0.02)
    @fact ROE_mean_a => roughly(0.091,0.0015)

    sale_phases = mean(results.sale_phases,1)
    @fact sale_phases[7] => roughly(1.1,0.1)
    @fact sale_phases[5]+sale_phases[6] => roughly(2.2,0.1)
    @fact sale_phases[4] => roughly(103.1,0.2)
end

##Basic test of geometric time distribution.
facts("Drug discovery Today Structured Equity") do
    reload("init_params_fn_DDT_struc_equity.jl")
    params=init_params_fn()
    params.simu.NSIMUS=NSIMUS
    results=simulate_CFs_fn(params,false,false)
 	LIQHORIZON    = results.params.simu.TIMESTEPS+int(ceil(maximum(results.params.assets[1].sale_time)))
    NGUARPERS  = size(results.guarantee_draw,2)
    NYEARS     = LIQHORIZON/results.params.simu.STEPS_YR
	
    ROE_raw        = results.ROE
    ROE_annualized = ((1+ROE_raw).^(1/NYEARS))-1
    ROE_mean       = mean(ROE_raw,1)
    ROE_mean_a     = mean(ROE_annualized,1)
    
    @fact ROE_mean[1]   => roughly(1.34,0.02)
    @fact ROE_mean[2]   => roughly(1.93,0.02)
    @fact ROE_mean_a[1] => roughly(0.102,0.0015)
    @fact ROE_mean_a[2] => roughly(0.1257,0.0015)

    sale_phases = mean(results.sale_phases,1)
    @fact sale_phases[5] => roughly(4.9,0.1)
    @fact sale_phases[4] => roughly(2.0,0.1)
    @fact sale_phases[3] => roughly(0.1,0.05)
end

##Basic test of geometric time distribution.
facts("Drug discovery Today Structured Equity Deterministic") do
    reload("init_params_fn_DDT_struc_equity_deterministicV.jl")
    params=init_params_fn()
    params.simu.NSIMUS=NSIMUS
    results=simulate_CFs_fn(params,false,false)
 	LIQHORIZON    = results.params.simu.TIMESTEPS+int(ceil(maximum(results.params.assets[1].sale_time)))
    NGUARPERS  = size(results.guarantee_draw,2)
    NYEARS     = LIQHORIZON/results.params.simu.STEPS_YR
	
    ROE_raw        = results.ROE
    ROE_annualized = ((1+ROE_raw).^(1/NYEARS))-1
    ROE_mean       = mean(ROE_raw,1)
    ROE_mean_a     = mean(ROE_annualized,1)
    
    @fact ROE_mean[1]   => roughly(0.7,0.02)
    @fact ROE_mean[2]   => roughly(2.52,0.02)
    @fact ROE_mean_a[1] => roughly(0.0730,0.0015)
    @fact ROE_mean_a[2] => roughly(0.1639,0.0015)

    sale_phases = mean(results.sale_phases,1)
    @fact sale_phases[5] => roughly(4.9,0.1)
    @fact sale_phases[4] => roughly(2.1,0.1)
    @fact sale_phases[3] => roughly(0.1,0.05)
end

